/*
    rox-media (Media) - DeviceKit-disks utility for ROX
    Copyright (C) 2009 Tony Houghton <h@realh.co.uk>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "i18n.h"
#include "icon_source.h"
#include "icon_source_filename.h"


static GdkPixbuf *
get_pixbuf(HIconSource *iself, int width, int height)
{
    HIconSourceFilename *self = H_ICON_SOURCE_FILENAME(iself);
    GdkPixbuf *pixbuf;
    GError *error = NULL;
    
    g_return_val_if_fail(self->filename, NULL);
    if (width <=0 && height <= 0)
        return NULL;
    pixbuf = gdk_pixbuf_new_from_file_at_scale(self->filename, width, height,
            TRUE, &error);
    if (!pixbuf)
    {
        g_warning(_("Unable to generate pixbuf from '%s': %s"),
                self->filename, error->message);
        g_error_free(error);
    }
    return pixbuf;
}

static void
h_icon_source_filename_init_h_icon_source(HIconSourceInterface *iface)
{
    iface->get_pixbuf = get_pixbuf;
}

G_DEFINE_TYPE_WITH_CODE(HIconSourceFilename,
        h_icon_source_filename, G_TYPE_OBJECT,
        G_IMPLEMENT_INTERFACE(H_TYPE_ICON_SOURCE,
                h_icon_source_filename_init_h_icon_source));


static void
h_icon_source_filename_init(HIconSourceFilename *self)
{
    self->filename = NULL;
}

static void
h_icon_source_filename_dispose(GObject *gobj)
{
    HIconSourceFilename *self = H_ICON_SOURCE_FILENAME(gobj);
    
    g_free(self->filename);
    self->filename = NULL;
    G_OBJECT_CLASS(h_icon_source_filename_parent_class)->dispose(gobj);
}

static void
h_icon_source_filename_finalize(GObject *gobj)
{
    G_OBJECT_CLASS(h_icon_source_filename_parent_class)->finalize(gobj);
}

static void
h_icon_source_filename_class_init(HIconSourceFilenameClass *klass)
{
    GObjectClass *gclass;
    

    gclass = G_OBJECT_CLASS(klass);
    gclass->dispose = h_icon_source_filename_dispose;
    gclass->finalize = h_icon_source_filename_finalize;
}

HIconSourceFilename *
h_icon_source_filename_new(const char *filename)
{
    HIconSourceFilename *self = H_ICON_SOURCE_FILENAME(g_object_new(
            H_TYPE_ICON_SOURCE_FILENAME, NULL));
    
    if (filename)
        h_icon_source_filename_set_filename(self, filename);
    return self;
}

void
h_icon_source_filename_set_filename(HIconSourceFilename *self,
        const char *filename)
{
    g_return_if_fail(H_IS_ICON_SOURCE_FILENAME(self));
    g_free(self->filename);
    self->filename = g_strdup(filename);
    g_signal_emit(self, h_icon_source_signals[H_ICON_SOURCE_SIGNAL_CHANGED], 0);
}
