/*
    rox-media (Media) - DeviceKit-disks utility for ROX
    Copyright (C) 2009 Tony Houghton <h@realh.co.uk>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <gdk-pixbuf/gdk-pixbuf.h>

#include "i18n.h"
#include "icon_source.h"
#include "icon_source_icon_name.h"
#include "icon_source_icon_name_priv.h"

#define H_ICON_SOURCE_ICON_NAME_GET_PRIVATE(obj) \
        (G_TYPE_INSTANCE_GET_PRIVATE((obj), \
        H_TYPE_ICON_SOURCE_ICON_NAME, HIconSourceIconNamePrivate))


static GdkPixbuf *
get_pixbuf(HIconSource *iself, int width, int height)
{
    HIconSourceIconName *self = H_ICON_SOURCE_ICON_NAME(iself);
    HIconSourceIconNamePrivate *priv = self->priv;
    GError *error = NULL;
    GdkPixbuf *pixbuf;

    g_return_val_if_fail(self->icon_name && priv->theme, NULL);
    if (width <=0 && height <= 0)
        return NULL;
    height = MAX(width, height);
    pixbuf = gtk_icon_theme_load_icon(priv->theme, self->icon_name, height,
            GTK_ICON_LOOKUP_FORCE_SIZE, &error);
    if (!pixbuf)
    {
        g_warning(_("Unable to generate pixbuf from icon name '%s': %s"),
                self->icon_name, error->message);
        g_error_free(error);
    }
    return pixbuf;
}

static void
h_icon_source_icon_name_init_h_icon_source(HIconSourceInterface *iface)
{
    iface->get_pixbuf = get_pixbuf;
}

G_DEFINE_TYPE_WITH_CODE(HIconSourceIconName,
        h_icon_source_icon_name, G_TYPE_OBJECT,
        G_IMPLEMENT_INTERFACE(H_TYPE_ICON_SOURCE,
                h_icon_source_icon_name_init_h_icon_source));


static void
h_icon_source_icon_name_init(HIconSourceIconName *self)
{

    self->priv = H_ICON_SOURCE_ICON_NAME_GET_PRIVATE(self);
    self->icon_name = NULL;
    self->priv->theme = NULL;
}

static void
h_icon_source_icon_name_dispose(GObject *gobj)
{
    HIconSourceIconName *self = H_ICON_SOURCE_ICON_NAME(gobj);
    HIconSourceIconNamePrivate *priv = self->priv;

    if (priv->theme)
    {
        g_object_unref(priv->theme);
        priv->theme = NULL;
    }
    g_free(self->icon_name);
    self->icon_name = NULL;
    G_OBJECT_CLASS(h_icon_source_icon_name_parent_class)->dispose(gobj);
}

static void
h_icon_source_icon_name_finalize(GObject *gobj)
{
    G_OBJECT_CLASS(h_icon_source_icon_name_parent_class)->finalize(gobj);
}

static void
h_icon_source_icon_name_class_init(HIconSourceIconNameClass *klass)
{
    GObjectClass *gclass;
    
    g_type_class_add_private(klass, sizeof(HIconSourceIconNamePrivate));

    gclass = G_OBJECT_CLASS(klass);
    gclass->dispose = h_icon_source_icon_name_dispose;
    gclass->finalize = h_icon_source_icon_name_finalize;
}

HIconSourceIconName *
h_icon_source_icon_name_new(GdkScreen *screen, const char *icon_name)
{
     HIconSourceIconName *self = H_ICON_SOURCE_ICON_NAME(
            g_object_new(H_TYPE_ICON_SOURCE_ICON_NAME, NULL));
    
    if (screen)
        h_icon_source_icon_name_set_screen(self, screen);
    if (icon_name)
        h_icon_source_icon_name_set_icon_name(self, icon_name);
    return self;
}

void
h_icon_source_icon_name_set_screen(HIconSourceIconName *self, GdkScreen *screen)
{
    HIconSourceIconNamePrivate *priv = self->priv;
    
    g_return_if_fail(H_IS_ICON_SOURCE_ICON_NAME(self));
    if (priv->theme)
        gtk_icon_theme_set_screen(priv->theme, screen);
    else
        priv->theme = gtk_icon_theme_get_for_screen(screen);
    g_signal_emit(self, h_icon_source_signals[H_ICON_SOURCE_SIGNAL_CHANGED], 0);
}

void
h_icon_source_icon_name_set_icon_name(HIconSourceIconName *self,
        const char *icon_name)
{
    g_return_if_fail(H_IS_ICON_SOURCE_ICON_NAME(self));
    g_free(self->icon_name);
    self->icon_name = g_strdup(icon_name);
    g_signal_emit(self, h_icon_source_signals[H_ICON_SOURCE_SIGNAL_CHANGED], 0);
}
