/*
    rox-media (Media) - DeviceKit-disks utility for ROX
    Copyright (C) 2009 Tony Houghton <h@realh.co.uk>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "panel_icon.h"

static void
source_changed(HPanelIcon *iself, HIconSource *icon_source)
{
    int width, height;
    HPanelIconData *data = H_PANEL_ICON_GET_INTERFACE(iself)->get_data(iself);
    GdkPixbuf *pixbuf;
    
    if (h_panel_icon_get_panel_orientation(iself) == GTK_ORIENTATION_VERTICAL)
    {
        width = data->current_size;
        height = -1;
    }
    else
    {
        width = -1;
        height = data->current_size;
    }
    pixbuf = h_icon_source_get_pixbuf(data->icon_source, width, height);
    if (pixbuf)
        h_panel_icon_show_pixbuf(iself, pixbuf);
}

GType
h_panel_icon_get_type(void)
{
    static GType iface_type = 0;
    
    if (!iface_type)
    {
        static const GTypeInfo info = {
            sizeof(HPanelIconInterface),
            NULL,   /* base_init */
            NULL,   /* base_finalize */
        };
    
        iface_type = g_type_register_static(G_TYPE_INTERFACE,
                "HPanelIconInterface", &info, 0);
    }
    return iface_type;
}

void
h_panel_icon_set_icon_source(HPanelIcon *iself, HIconSource *icon_source)
{
    g_return_if_fail(H_IS_PANEL_ICON(iself));
    H_PANEL_ICON_GET_INTERFACE(iself)->set_icon_source(iself, icon_source);
}

void
h_panel_icon_destroy(HPanelIcon *iself)
{
    g_return_if_fail(H_IS_PANEL_ICON(iself));
    H_PANEL_ICON_GET_INTERFACE(iself)->destroy(iself);
}

void
h_panel_icon_set_tooltip_text(HPanelIcon *iself, const char *text)
{
    g_return_if_fail(H_IS_PANEL_ICON(iself));
    H_PANEL_ICON_GET_INTERFACE(iself)->set_tooltip_text(iself, text);
}

char *
h_panel_icon_get_tooltip_text(HPanelIcon *iself)
{
    g_return_val_if_fail(H_IS_PANEL_ICON(iself), NULL);
    return H_PANEL_ICON_GET_INTERFACE(iself)->get_tooltip_text(iself);
}

void
h_panel_icon_set_tooltip_markup(HPanelIcon *iself, const char *text)
{
    g_return_if_fail(H_IS_PANEL_ICON(iself));
    H_PANEL_ICON_GET_INTERFACE(iself)->set_tooltip_markup(iself, text);
}

char *
h_panel_icon_get_tooltip_markup(HPanelIcon *iself)
{
    g_return_val_if_fail(H_IS_PANEL_ICON(iself), NULL);
    return H_PANEL_ICON_GET_INTERFACE(iself)->get_tooltip_markup(iself);
}

void
h_panel_icon_size_set(HPanelIcon *iself, int width, int height)
{
    g_return_if_fail(H_IS_PANEL_ICON(iself));
    H_PANEL_ICON_GET_INTERFACE(iself)->size_set(iself, width, height);
}

void
h_panel_icon_show_pixbuf(HPanelIcon *iself, GdkPixbuf *pixbuf)
{
    g_return_if_fail(H_IS_PANEL_ICON(iself));
    H_PANEL_ICON_GET_INTERFACE(iself)->show_pixbuf(iself, pixbuf);
}

GtkOrientation
h_panel_icon_get_panel_orientation(HPanelIcon *iself)
{
    g_return_val_if_fail(H_IS_PANEL_ICON(iself), GTK_ORIENTATION_HORIZONTAL);
    return H_PANEL_ICON_GET_INTERFACE(iself)->get_panel_orientation(iself);
}

GtkMenuPositionFunc
h_panel_icon_get_menu_position_func(HPanelIcon *iself)
{
    g_return_val_if_fail(H_IS_PANEL_ICON(iself), NULL);
    return H_PANEL_ICON_GET_INTERFACE(iself)->get_menu_position_func(iself);
}
    
HPanelIconData *
h_panel_icon_get_data(HPanelIcon *iself)
{
    g_return_val_if_fail(H_IS_PANEL_ICON(iself), NULL);
    return H_PANEL_ICON_GET_INTERFACE(iself)->get_data(iself);
}

void
h_panel_icon_data_init(HPanelIconData *data)
{
    data->icon_source = NULL;
    data->current_size = -1;
}
    
void
h_panel_icon_data_dispose(HPanelIconData *data)
{
    if (data->icon_source)
    {
        g_object_unref(data->icon_source);
        data->icon_source = NULL;
    }
}
    

void
h_panel_icon_standard_set_icon_source(HPanelIcon *iself,
        HIconSource *icon_source)
{
    HPanelIconData *data;
    
    g_return_if_fail(H_IS_ICON_SOURCE(icon_source));
    data = H_PANEL_ICON_GET_INTERFACE(iself)->get_data(iself);
    if (data->icon_source)
        g_object_unref(data->icon_source);
    data->icon_source = icon_source;
    g_object_ref_sink(icon_source);
    if (data->current_size != -1)
    {
        GdkPixbuf *pixbuf;
        int width, height;
        if (h_panel_icon_get_panel_orientation(iself) ==
                GTK_ORIENTATION_VERTICAL)
        {
            width = data->current_size;
            height = -1;
        }
        else
        {
            width = -1;
            height = data->current_size;
        }
        pixbuf = h_icon_source_get_pixbuf(icon_source, width, height);
        if (pixbuf)
            h_panel_icon_show_pixbuf(iself, pixbuf);
    }
    g_signal_connect_swapped(icon_source, "changed",
            G_CALLBACK(source_changed), iself);
}

void
h_panel_icon_standard_size_set(HPanelIcon *iself, int width, int height)
{
    HPanelIconData *data = H_PANEL_ICON_GET_INTERFACE(iself)->get_data(iself);
    int new_size;
    
    if (!data->icon_source)
        return;
    if (width > 1)
        width -= 2;
    if (height > 1)
        height -= 2;
    if (h_panel_icon_get_panel_orientation(iself) == GTK_ORIENTATION_VERTICAL)
        new_size = width;
    else
        new_size = height;
    if (new_size != data->current_size)
    {
        GdkPixbuf *pixbuf = h_icon_source_get_pixbuf(data->icon_source,
                width, height);
                
        if (pixbuf)
            h_panel_icon_show_pixbuf(iself, pixbuf);
        data->current_size = new_size;
    }
}
