#ifndef __H_PANEL_ICON_H
#define __H_PANEL_ICON_H
/*
    rox-media (Media) - DeviceKit-disks utility for ROX
    Copyright (C) 2009 Tony Houghton <h@realh.co.uk>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

/* Interface for an icon which shows a graphic (provided by a HIconSource)
 * in a panel (ROX panel or system/status tray)
 */

#include <glib-object.h>
#include <gtk/gtk.h>

#include "icon_source.h"

G_BEGIN_DECLS

/* Data used by implementation; best embedded in implementation's priv */
typedef struct _HPanelIconData {
    HIconSource *icon_source;
    int current_size;
} HPanelIconData;

#define H_TYPE_PANEL_ICON \
        (h_panel_icon_get_type())
#define H_PANEL_ICON(obj) \
        (G_TYPE_CHECK_INSTANCE_CAST((obj), \
        H_TYPE_PANEL_ICON, HPanelIcon))
#define H_IS_PANEL_ICON(obj) \
        (G_TYPE_CHECK_INSTANCE_TYPE((obj), \
        H_TYPE_PANEL_ICON))
#define H_PANEL_ICON_GET_INTERFACE(inst) \
        (G_TYPE_INSTANCE_GET_INTERFACE((inst), \
        H_TYPE_PANEL_ICON, HPanelIconInterface))

typedef struct _HPanelIcon          HPanelIcon;
typedef struct _HPanelIconInterface HPanelIconInterface;

struct _HPanelIconInterface {
    GTypeInterface parent_iface;
    void (*set_icon_source)(HPanelIcon *iself, HIconSource *icon_source);
    void (*destroy)(HPanelIcon *iself);
    void (*set_tooltip_text)(HPanelIcon *iself, const char *text);
    char *(*get_tooltip_text)(HPanelIcon *iself);
    void (*set_tooltip_markup)(HPanelIcon *iself, const char *text);
    char *(*get_tooltip_markup)(HPanelIcon *iself);
    void (*size_set)(HPanelIcon *iself, int width, int height);
    void (*show_pixbuf)(HPanelIcon *iself, GdkPixbuf *pixbuf);
    GtkOrientation (*get_panel_orientation)(HPanelIcon *iself);
    GtkMenuPositionFunc (*get_menu_position_func)(HPanelIcon *iself);
    HPanelIconData *(*get_data)(HPanelIcon *iself);
};

GType
h_panel_icon_get_type(void) G_GNUC_CONST;


void
h_panel_icon_set_icon_source(HPanelIcon *iself, HIconSource *icon_source);

/* Use this because g_object_unref seems to break due to container/child */
void
h_panel_icon_destroy(HPanelIcon *iself);

void
h_panel_icon_set_tooltip_text(HPanelIcon *iself, const char *text);

char *
h_panel_icon_get_tooltip_text(HPanelIcon *iself);

void
h_panel_icon_set_tooltip_markup(HPanelIcon *iself, const char *text);

char *
h_panel_icon_get_tooltip_markup(HPanelIcon *iself);

/* The rest of these are typically only called by HPanelIcon implementations */

void
h_panel_icon_size_set(HPanelIcon *iself, int width, int height);

void
h_panel_icon_show_pixbuf(HPanelIcon *iself, GdkPixbuf *pixbuf);

GtkOrientation
h_panel_icon_get_panel_orientation(HPanelIcon *iself);

GtkMenuPositionFunc
h_panel_icon_get_menu_position_func(HPanelIcon *self);
    
HPanelIconData *
h_panel_icon_get_data(HPanelIcon *self);

/* Must be called from implementations' init and dispose functions
 * respectively
 */
void
h_panel_icon_data_dispose(HPanelIconData *data);

void
h_panel_icon_data_init(HPanelIconData *data);


/* Default functions which should suit most implementations */

void
h_panel_icon_standard_set_icon_source(HPanelIcon *iself,
        HIconSource *icon_source);

void
h_panel_icon_standard_size_set(HPanelIcon *iself, int width, int height);

G_END_DECLS

#endif /* __H_PANEL_ICON_H */
