/*
    rox-media (Media) - DeviceKit-disks utility for ROX
    Copyright (C) 2009 Tony Houghton <h@realh.co.uk>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <string.h>

#include <X11/Xlib.h>
#include <X11/Xatom.h>

#include <gdk/gdkx.h>

#include "rox_panel_info.h"

GtkPositionType
rox_panel_info_get_position(GtkWidget *plug)
{
    static gboolean position_known = FALSE;
    static GtkPositionType position = GTK_POS_BOTTOM;
    GdkAtom a_pos;
    Window xwin;
    GdkDisplay *gdisplay;
    Display *xdisplay;

    if (position_known)
        return position;

    /* Even if we don't find position, assume we did because if we couldn't
     * find it the first time, calling it again won't help */
    position_known = TRUE;

    /* Atom for panel pos property */
    a_pos = gdk_atom_intern("_ROX_PANEL_MENU_POS", FALSE);

    /* Get underlying X stuff */
    gtk_widget_realize(plug);
    gdisplay = gtk_widget_get_display(plug);
    if (!gdisplay)
        gdisplay = gdk_display_get_default();
    xdisplay = gdk_x11_display_get_xdisplay(gdisplay);
    xwin = GDK_WINDOW_XWINDOW(plug->window);

    /* Plug won't necessarily have the property we want, so work our way up
     * until we get to a window that does */
    do
    {
        Atom actual_type;
        int actual_format;
        unsigned long n_read, n_remain;
        unsigned char *prop_data = NULL;
        Window parent, root;
        Window *children;
        unsigned int n_children;
        
        if (XGetWindowProperty(xdisplay, xwin,
                    gdk_x11_atom_to_xatom(a_pos), 0, -1, FALSE, XA_STRING,
                    &actual_type, &actual_format, &n_read, &n_remain,
                    &prop_data)
                == Success && actual_format != 0)
        {
            if (!strncmp((const char *) prop_data, "Top", 3))
                position = GTK_POS_TOP;
            /* Don't really need to test for bottom, because it's default */
            /*
            else if (!strncmp((const char *) prop_data, "Bottom", 6))
                position = GTK_POS_BOTTOM;
            */
            else if (!strncmp((const char *) prop_data, "Left", 4))
                position = GTK_POS_LEFT;
            else if (!strncmp((const char *) prop_data, "Right", 5))
                position = GTK_POS_RIGHT;
            XFree(prop_data);
            break;
        }
        if (prop_data)
            XFree(prop_data);

        /* Haven't found property, so try next window up */
        children = NULL;
        if (!XQueryTree(xdisplay, xwin, &root, &parent, &children, &n_children))
        {
            xwin = 0;
        }
        else
        {
            xwin = parent;
        }
        if (children)
            XFree(children);
            
    } while (xwin);

    return position;
}

GtkOrientation
rox_panel_info_get_orientation(GtkWidget *plug)
{
    switch (rox_panel_info_get_position(plug))
    {
        case GTK_POS_RIGHT:
        case GTK_POS_LEFT:
            return GTK_ORIENTATION_VERTICAL;
            break;
        default:
            break;
    }
    return GTK_ORIENTATION_HORIZONTAL;
}

void
rox_panel_info_get_icon_position(GtkWidget *plug,
        GdkScreen **screen, int *monitor,
        int *root_x, int *root_y, int *width, int *height)
{
    GdkScreen *scrn;
    int x, y;
    
    if (screen || monitor)
        scrn = gtk_widget_get_screen(plug);
    if (screen)
        *screen = scrn;
    if (monitor || root_x || root_y)
        gdk_window_get_origin(plug->window, &x, &y);
    if (monitor)
        *monitor = gdk_screen_get_monitor_at_point(scrn, x, y);
    if (root_x)
        *root_x = x;
    if (root_y)
        *root_y = y;
    if (width || height)
        gdk_drawable_get_size(GDK_DRAWABLE(plug->window), width, height);
}
